@@ -1,6 +1,7 @@
 C*GREXEC -- PGPLOT device handler dispatch routine
 C 12/93 C. T. Dum: version for MS F32 Power Station
 C  20-Apr-1996: "W9DRIV", use CASE structure [PAS]
+C  10-Jul-1996: add GIF driver (already had PostScript, Null, LaTex) [PAS]
 C+
       SUBROUTINE GREXEC(IDEV, IFUNC, RBUF, NBUF, CHR, LCHR)
       INTEGER IDEV, IFUNC, NBUF, LCHR
@@ -8,7 +9,7 @@ C+
       CHARACTER*(*) CHR
 C---
       INTEGER NDEV
-      PARAMETER (NDEV=11)
+      PARAMETER (NDEV=13)
       CHARACTER*10 MSG
 C---
       SELECT CASE (IDEV)
@@ -38,6 +39,10 @@ C---
       CASE (10)
          CALL PSDRIV(IFUNC,RBUF,NBUF,CHR,LCHR,4)
       CASE (11)
+         CALL GIDRIV(IFUNC,RBUF,NBUF,CHR,LCHR,1)
+      CASE (12)
+         CALL GIDRIV(IFUNC,RBUF,NBUF,CHR,LCHR,2)
+      CASE (13)
          CALL LXDRIV(IFUNC,RBUF,NBUF,CHR,LCHR)
       CASE DEFAULT
          WRITE (MSG,'(I10)') IDEV
